@@ -239,6 +239,9 @@ class UnixExecutable(object):
     def is_executable(self):
         return True
 
+    def get_runpaths(self):
+        return self.dt_runpath
+
 
 def read_data(file, endian, num=1):
     """
@@ -474,6 +477,7 @@ class machofile(UnixExecutable):
     def __init__(self, file, arch, initial_rpaths_transitive=[]):
         self.filename = file.name
         self.shared_libraries = []
+        self.dt_runpath = []
         # Not actually used ..
         self.selfdir = os.path.dirname(file.name)
         results = mach_o_find_dylibs(file, arch)
@@ -928,6 +932,9 @@ class inscrutablefile(UnixExecutable):
     def get_resolved_shared_libraries(self, *args, **kw):
         return []
 
+    def get_runpaths(self):
+        return []
+
     def selfdir(self):
         return None
 
@@ -976,6 +983,14 @@ def is_codefile(filename, skip_symlinks=True):
     return True
 
 
+def codefile_type(filename, skip_symlinks=True):
+    "Returns None, 'machofile' or 'elffile'"
+    klass = codefile_class(filename, skip_symlinks=skip_symlinks)
+    if not klass:
+        return None
+    return klass.__name__
+
+
 def _trim_sysroot(sysroot):
     while sysroot.endswith('/') or sysroot.endswith('\\'):
         sysroot = sysroot[:-1]
@@ -1045,6 +1060,15 @@ def inspect_rpaths(filename, resolve_dirnames=True, use_os_varnames=True,
                 return cf.rpaths_nontransitive
 
 
+def get_runpaths(filename, arch='native'):
+    if not os.path.exists(filename):
+        return []
+    arch = _get_arch_if_native(arch)
+    with open(filename, 'rb') as f:
+        cf = codefile(f, arch, ['/lib', '/usr/lib'])
+        return cf.get_runpaths()
+
+
 # TODO :: Consider returning a tree structure or a dict when recurse is True?
 def inspect_linkages(filename, resolve_filenames=True, recurse=True, sysroot='', arch='native'):
     already_seen = set()
