@@ -4,7 +4,6 @@ import base64
 from collections import defaultdict
 import contextlib
 import fnmatch
-from glob2 import glob
 import json
 from locale import getpreferredencoding
 import logging
@@ -42,12 +41,23 @@ from conda_build.conda_interface import rm_rf as _rm_rf # NOQA
 from conda_build.os_utils import external
 
 if PY3:
+    from glob import glob as glob_glob
+
+    # stdlib glob is less feature-rich but considerably faster than glob2
+    def glob(pathname, recursive=True):
+        return glob_glob(pathname, recursive=recursive)
+
     import urllib.parse as urlparse
     import urllib.request as urllib
     # NOQA because it is not used in this file.
     from contextlib import ExitStack  # NOQA
     PermissionError = PermissionError  # NOQA
 else:
+    from glob2 import glob as glob2_glob
+
+    def glob(pathname, recursive=True):
+        return glob2_glob(pathname, recursive=recursive)
+
     import urlparse
     import urllib
     # NOQA because it is not used in this file.
@@ -801,7 +811,12 @@ def get_stdlib_dir(prefix, py_ver):
     if sys.platform == 'win32':
         lib_dir = os.path.join(prefix, 'Lib')
     else:
-        lib_dir = os.path.join(prefix, 'lib', 'python{}'.format(py_ver))
+        lib_dir = os.path.join(prefix, 'lib')
+        python_folder = glob(os.path.join(lib_dir, 'python?.*'))
+        if python_folder:
+            lib_dir = os.path.join(lib_dir, python_folder[0])
+        else:
+            lib_dir = os.path.join(lib_dir, 'python{}'.format(py_ver))
     return lib_dir
 
 
